#!/usr/bin/perl -w
#NAME ESXHealth.pl  version 1.1
#Created by Shannon Anderson taking from examples in getESXHealthreport.pl available on VMware.com
#Date 11/12/12
#WEB  www.squidworks.net
#
# Dependencies: Requires VSphere Perl SDK installed and CURL
# If windows you can install curl for windows or use the portable executible provided in zip.
#
#usage = C:\>perl getESXHostHealth.pl --server myfully.qualified.domain.name --username root --password mypassword
#Schedule Windows task (Copy to commandline and edit line for correct information
#   schtasks /create /RU "my-username" /RP "my-password"  /tn "ESX Health" /tr "C:\Dir where script is\ESXHealth.pl --server Fully.Qualified.Doamin.Name --username root --password my-root-password" /sc MINUTE /mo 15 

use VMware::VIRuntime;
use VMware::VILib;
use Net::SMTP;

Opts::add_options(%opts);
Opts::parse();
Opts::validate();
Util::connect();
my $host_view = Vim::find_entity_view(view_type => 'HostSystem');
&getHardwareHealthInfo($host_view);
Util::disconnect();


#--------------------------------------
#
# Subs Start Here
#
#
sub getHardwareHealthInfo {
	my $debug = 0;
	my $REPORT_OUTPUT = "";
	my $curloutput = "";
	my $CurlOptions = "-H \"Content-type: application/octet-stream\" -H \"Mime-version: 1.0\""; 
# Change URL to pint at your API for capturing	
	my $MAGURL = "http://user:password\@my.webserver.com/xymon-cgi/xymoncgimsg.cgi";
	my $host_name = Opts::get_option('server');
	my ($host_view) = @_;
	my $hardwareSystem = Vim::get_view(mo_ref => $host_view->configManager->healthStatusSystem);
	my ($cpu,$mem,$storage,@sensors);
    my ($logsec,$logmin,$loghour,$logmday,$logmon,$logyear,$logwday,$logyday,$logisdst)=localtime(time);
	my $timestamp = sprintf("%02d/%02d/%4d %02d:%02d:%02d",$logmon+1,$logmday,$logyear+1900,$loghour,$logmin,$logsec);
	if($hardwareSystem->runtime->hardwareStatusInfo) {	
			###########
			# CPU
			###########
			my $cpuStatus = $hardwareSystem->runtime->hardwareStatusInfo->cpuStatusInfo;
			foreach(@$cpuStatus) {
				if($_->status->key ne 'Green' || $debug eq 1) {
					$cpu .= "\t\t<div id=\"content-main\" style=\"color:" . $_->status->key ."\"><p>". $_->name ."</p></div>\n";
				}
			}

			###########
			# MEMORY
			###########
			my $memStatus = $hardwareSystem->runtime->hardwareStatusInfo->memoryStatusInfo;
			foreach(@$memStatus) {
				if($_->status->key ne 'Green' || $debug eq 1) {
					$mem .= "\t\t<div id=\"content-main\" style=\"color:" . $_->status->key ."\"><p>". $_->name ."</p></div>\n";
       	   	             }
			}

			###########
			# STORAGE 
			###########
			my $stoStatus = $hardwareSystem->runtime->hardwareStatusInfo->storageStatusInfo;
			foreach(@$stoStatus) {
				if($_->status->key ne 'Green' || $debug eq 1) {
					$storage .= "\t\t<div id=\"content-main\" style=\"color:" . $_->status->key ."\"><p>". $_->name ."</p></div>\n";
                       		}
               		}
	}
			
	if($hardwareSystem->runtime->systemHealthInfo) {
			##########################
			# OTHER SYSTEM COMPONENTS
			##########################
			my $sensorInfo = $hardwareSystem->runtime->systemHealthInfo->numericSensorInfo;
			foreach(@$sensorInfo) {
				if($_->healthState && $_->healthState->label ne 'Green' || $debug eq 1) {
					my $reading = $_->currentReading * $_->unitModifier; 
					my $units;
					if($_->rateUnits) {
						$units = $_->baseUnits . "/" . $_->rateUnits;
					} else { $units = $_->baseUnits; }
					my $sensor_string = $_->sensorType . "==" . "\t\t<div id=\"content-main\" style=\"color:" . $_->healthState->key ."\"><p>". $_->name ." --- ". $reading ." ". $units ."</p></div>\n";
					push @sensors,$sensor_string;
				}
			}
	}

		##################
		# PRINT SUMMARY
		##################

		#everything A okay
		my $build = $host_view->summary->config->product->fullName if($host_view->summary->config->product->fullName);
		if(!$cpu && !$mem && !$storage && !@sensors) {
			$REPORT_OUTPUT = "\"status $host_name.esxhost green ";
			$REPORT_OUTPUT .= $timestamp;			
			$REPORT_OUTPUT .= "\n\n\n&green $host_name ($build) Health is Good!\n";			
			
             	} else {
			$REPORT_OUTPUT = "\"status $host_name.esxhost red ";
			$REPORT_OUTPUT .= $timestamp;	
			
			$REPORT_OUTPUT .= "\n\n\n&red $host_name ($build) Health is Failing!!!\n";		
		}

		#something was bad
		if($cpu) { 
			$REPORT_OUTPUT .= "\t<div id=\"content\"><p>CPU COMPONENTS</p></div>\n";
			$REPORT_OUTPUT .= $cpu;
		}
		if($mem) { 
			$REPORT_OUTPUT .= "\t<div id=\"content\"><p>MEMORY COMPONENTS</p></div>\n";
			$REPORT_OUTPUT .= $mem; 
		}
		if($storage) { 
			$REPORT_OUTPUT .= "\t<div id=\"content\"><p>STORAGE COMPONENTS</p></div>\n";
			$REPORT_OUTPUT .= $storage; 
		}
		if(@sensors) {
			my %seen;
			foreach(@sensors) {
				my ($component,$sensor) = split('==',$_);
				if(!$seen{$component}) {
					$seen{$component} = "yes";
					my $type = uc $component;
					$REPORT_OUTPUT .= "\t<div id=\"content\"><p>$type COMPONENTS</p></div>\n";
				}
				if($seen{$component}) { 
					$REPORT_OUTPUT .= $sensor;
				}
			}
		}
		$REPORT_OUTPUT .= "\n\n\"";

	
	$curloutput = `curl $CurlOptions $MAGURL  -d  $REPORT_OUTPUT`;

# Uncomment if using windows and want a log of when the script runs and what it spit out.
#	open(MYLOG, "> C:/temp/esxhost.log");
#		print MYLOG "$REPORT_OUTPUT\n\n$curloutput\n\n\n";
#	close(MYLOG);
}